﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Principal;
using Orchard;
using Orchard.ContentManagement;
using Orchard.Data;
using Orchard.Environment.Extensions;
using Orchard.Localization;
using Orchard.Logging;
using Orchard.Mvc;
using Orchard.Roles.Models;
using Orchard.Roles.Services;
using Orchard.Security;
using Orchard.Services;
using Orchard.UI.Notify;
using Orchard.Users.Events;

namespace AlexZh.WindowsAuthentication.Services
{
	[OrchardSuppressDependency("Orchard.Security.Providers.FormsAuthenticationService")]
	public class CustomAuthenticationService : IAuthenticationService
	{
		private readonly FormsAuthenticationService systemAuthenticationService;
		private readonly IMembershipService membershipService;
		private readonly IHttpContextAccessor httpContextAccessor;
		private readonly IRoleService roleService;
		private readonly IRepository<UserRolesPartRecord> userRolesRepository;
		private readonly IWinAuthSettingsService winUserSettingsService;
		private readonly IOrchardServices orchardServices;
		private readonly IEnumerable<IUserEventHandler> userEventHandlers;

		private ILogger logger;

		public ILogger Logger
		{
			get { return logger; }
			set
			{
				logger = value;
				systemAuthenticationService.Logger = logger;
			}
		}

		public Localizer T { get; set; }

		public CustomAuthenticationService(
			IClock clock,
			IContentManager contentManager,
			IMembershipService membershipService,
			IRoleService roleService,
			IHttpContextAccessor httpContextAccessor,
			IRepository<UserRolesPartRecord> userRolesRepository,
			IWinAuthSettingsService winUserSettingsService,
			IOrchardServices orchardServices,
			IEnumerable<IUserEventHandler> userEventHandlers)
		{
			this.membershipService = membershipService;
			this.roleService = roleService;
			this.httpContextAccessor = httpContextAccessor;
			this.userRolesRepository = userRolesRepository;
			this.winUserSettingsService = winUserSettingsService;
			this.orchardServices = orchardServices;
			this.userEventHandlers = userEventHandlers;
			systemAuthenticationService = new FormsAuthenticationService(clock, contentManager, httpContextAccessor);

			Logger = NullLogger.Instance;
			T = NullLocalizer.Instance;
		}

		#region UserName from WindowsIdentity

		private static string GetUserName(IIdentity identity)
		{
			return identity == null ? null : GetUserName(identity.Name);
		}

		private static string GetUserName(string identityName)
		{
			if (string.IsNullOrEmpty(identityName))
				return identityName;
			int index = identityName.IndexOf('\\');
			if (index > 0 && index < (identityName.Length - 1))
			{
				identityName = identityName.Substring(index + 1);
			}
			index = identityName.IndexOf('@');
			if (index > 0 && index < (identityName.Length - 1))
			{
				identityName = identityName.Remove(index);
			}

			return identityName;
		}

		#endregion

		#region Implementation of IAuthenticationService

		public void SignIn(IUser user, bool createPersistentCookie)
		{
			systemAuthenticationService.SignIn(user, createPersistentCookie);
		}

		public void SignOut()
		{
			systemAuthenticationService.SignOut();
		}

		public void SetAuthenticatedUserForRequest(IUser user)
		{
			systemAuthenticationService.SetAuthenticatedUserForRequest(user);
		}

		public IUser GetAuthenticatedUser()
		{
			// Try to use default authentication seervice
			var user = systemAuthenticationService.GetAuthenticatedUser();
			if (user == null)
			{
				var httpContext = httpContextAccessor.Current();

				if (httpContext != null && httpContext.Request.IsAuthenticated && (httpContext.User.Identity is WindowsIdentity))
				{
					// Windows authentication
					var settings = winUserSettingsService.Retrieve();
					if (!settings.EnableWindowsAuthentication)
					{
						orchardServices.Notifier.Add(NotifyType.Error, T("Windows authentication is disabled. Please contact administrator."));
						return null;
					}

					var windowsIdentity = (WindowsIdentity) httpContext.User.Identity;
					string userName = GetUserName(windowsIdentity);
					user = membershipService.GetUser(userName);

					// Creating new user if not exists
					if (user == null)
					{
						string email = settings.EmailDomain;
						if (string.IsNullOrEmpty(email))
							email = null;
						else
							email = userName + "@" + email;

						user = membershipService.CreateUser(new CreateUserParams(
						                                    	userName,
						                                    	Guid.NewGuid().ToString(),
						                                    	email,
						                                    	Guid.NewGuid().ToString(),
						                                    	Guid.NewGuid().ToString(),
						                                    	true));
						// Adding default roles to user
						if (user != null)
						{
							var roles = settings.DefaultRoles.ToArray();

							foreach (var role in roles)
							{
								var roleRecord = roleService.GetRoleByName(role);
								if (roleRecord != null)
									userRolesRepository.Create(new UserRolesPartRecord {Role = roleRecord, UserId = user.Id});
							}
						}
					}
					// Signing In Windows User
					if (user != null)
					{
						SetAuthenticatedUserForRequest(user);
						SignIn(user, false);
						foreach (var userEventHandler in userEventHandlers)
						{
							userEventHandler.LoggedIn(user);
						}
					}
				}
			}

			return user;
		}

		#endregion
	}
}